@@ -16,7 +16,17 @@ from datetime import datetime, timedelta
 from decimal import Decimal
 from enum import Enum
 from types import MappingProxyType
-from typing import Any, Callable, Dict, Optional, Tuple, TypeVar, Union, get_type_hints
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    get_type_hints,
+)
 
 __all__ = ("json_options", "JSONAble", "JSON", "J")
 
@@ -113,6 +123,12 @@ class JSONAble:
     # json_options (if declared), and then in the class-level `__default_json_options__`.
     __default_json_options__ = json_options()
 
+    def _get_origin_json(self) -> JSON:
+        """Debug purpose method to return the original JSON dictionary which constructs this instance via
+        `from_json` method.
+        """
+        return getattr(self, "__dataclass_origin_json__")
+
     @classmethod
     def get_encoder(cls, t) -> F:
         """Returns an encoder function for given type `t`.
@@ -144,6 +160,15 @@ class JSONAble:
         elif _is_jsonable_like(t):
             # Nested
             return _encode_jsonable
+        elif t is list:
+            # list of elements.
+            return lambda x: [cls.get_encoder(type(e))(e) for e in x]
+        elif t is set:
+            # set of elements
+            return lambda x: {cls.get_encoder(type(e))(e) for e in x}
+        elif t is dict:
+            # dict
+            return _encode_dict
         elif _is_generics(t) and _get_generics_origin(t) in {list, set}:
             # List[E] / Set[E]
             args = _get_generics_args(t)
@@ -201,6 +226,15 @@ class JSONAble:
         elif t is Any:
             # Any returns reflection decoding.
             return lambda x: cls.get_decoder(type(x))(x)
+        elif t is list:
+            # list of elements.
+            return lambda x: [cls.get_decoder(type(e))(e) for e in x]
+        elif t is set:
+            # set of elements
+            return lambda x: {cls.get_decoder(type(e))(e) for e in x}
+        elif t is dict:
+            # dict
+            return _encode_dict
         elif isinstance(t, type) and issubclass(t, Enum):
             return t
         elif _is_jsonable_like(t):
@@ -312,8 +346,11 @@ class JSONAble:
 
         return d
 
+    # An alias for `json`
+    to_json = json
+
     @classmethod
-    def from_json(cls, d: JSON) -> T:
+    def from_json(cls: Type[T], d: JSON) -> T:
         """Constructs an instance of this dataclass from given jsonable dictionary."""
 
         # Arguments for class `cls()`.
@@ -362,7 +399,9 @@ class JSONAble:
             decoder = options.decoder or cls.get_decoder(t)
             kwds[name] = decoder(v)
 
-        return cls(**kwds)  # type: ignore
+        inst = cls(**kwds)
+        setattr(inst, "__dataclass_origin_json__", d)
+        return inst  # type: ignore
 
 
 J = JSONAble  # short alias
@@ -382,6 +421,21 @@ _decode_decimal = lambda x: Decimal(str(x))
 
 _default_omitempty_tester = lambda x: not x
 
+
+def _encode_dict(x):
+    for k in x:
+        if not isinstance(k, str):
+            raise NotImplementedError("dict with non-str keys is not supported")
+    return {k: JSONAble.get_encoder(type(v))(v) for k, v in x.items()}
+
+
+def _decode_dict(x):
+    for k in x:
+        if not isinstance(k, str):
+            raise NotImplementedError("dict with non-str keys is not supported")
+    return {k: JSONAble.get_decoder(type(v))(v) for k, v in x.items()}
+
+
 # Utils
 def _is_generics(t) -> bool:
     """Returns whether the given type `t` is a generics type."""
